#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2024/3/11 11:29
# @Author  : 杨智杰
# @Email   : yangzhijie@datagrand.com
import json
import jsonpath
from dg_itest.utils.logger import log_attach
from dg_itest.utils.diff_helper import DiffHelper
from dg_itest.utils.cache import local_cache

class DgAfterHandler:
	def __init__(self):
		pass

	# todo 断言类型需要增加，目前只支持eq(相等), co(包含), sa(存储)。
	def after(self, in_params, response):
		'''
		对接口返回值进行各种后处理操作：断言，提取值，判断包含
		in_params: 传入的参数，格式为json
		response: 接口返回的http响应
		'''
		validate = in_params.get("validate")
		expect = in_params.get('expect')
		for item in validate:
			if "sa" in item.keys():  # save临时存储，后面用例可用
				self._save(item, response)
			if "eq" in item.keys():
				self._equal(item, response, expect)
			if "co" in item.keys():
				self._contains(item, response, expect)


	def _equal(self, item, response, expect):
		'''
		item: 单个用例的入参
		response: 接口返回的http响应
		expect：用例执行的期望值
		'''
		validate_rule = item.get("eq")
		actual_result = jsonpath.jsonpath(response.json(), validate_rule)
		expect_result = jsonpath.jsonpath(expect.get('json'), validate_rule)

		if isinstance(actual_result, list) and isinstance(expect_result, list):
			actual_result = sorted(actual_result)
			expect_result = sorted(expect_result)
		log_attach(f'equal check --> validate_rule: {validate_rule} ,\nactual_result: {actual_result}\nexpect_result: {expect_result}', name='equal assert')
		assert actual_result == expect_result, '\n' + DiffHelper.diff(str(actual_result), str(expect_result))


	def _contains(self, item, response, expect):
		'''
		item: 单个用例的入参
		response: 接口返回的http响应
		expect：用例执行的期望值
		'''
		contains_rule = item.get('co')
		actual_result = jsonpath.jsonpath(response.json(), contains_rule)
		expect_result = expect.get('contains')
		log_attach(
			f'contains check --> validate_rule: {contains_rule} ,\nactual_result: {actual_result}\nexpect_result: {expect_result}',
			name='contains assert')
		assert expect_result in json.dumps(actual_result), f'{expect_result} not in {actual_result}'

	def _save(self, item, response):
		'''
		item: 单个用例的入参
		response: 接口返回的http响应
		'''
		sa_value = item.get("sa")
		for sa_item_key in sa_value.keys():
			sa_item_value = jsonpath.jsonpath(response.json(), sa_value.get(sa_item_key))
			assert type(sa_item_value) is list and len(sa_item_value) > 0, '\n' + '未获取到值'
			sa_item_keep_value = eval(f'{item.get("convert")}({sa_item_value[0]})') if item.get("convert") else sa_item_value[0]
			log_attach(f'local cache --> ${sa_item_key}$:{sa_item_keep_value} , cache type: {str(type(sa_item_keep_value))}', name='save local cache')
			local_cache.put(f"${sa_item_key}$", sa_item_keep_value)


	def check_status(self, in_params):
		"""
        校验接口返回状态
        由于接口存在异步执行，返回对应状态耗时较长，通过其状态校验，多次循环执行得到正确实际状态
        :return: 返回状态
        """
		status = True
		for item in in_params.get("validate"):
			if 'cs' in item.keys():
				cs_value = item.get('cs')
				for cs_item_key in cs_value.keys():
					status = status and local_cache[f"${cs_item_key}$"] == cs_value.get(cs_item_key)
		return status



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2024/3/11 17:41
# @Author  : 杨智杰
# @Email   : yangzhijie@datagrand.com

from pathlib import Path
from dg_itest import local_test_res

class FileHelper:

	@staticmethod
	def replace_file_to_stream(files_array):
		"""
        替换文件为文件流
        """
		all_resource_files = Path(local_test_res).rglob("*.*")
		files_buffer = []
		for file_name in files_array:
			file = [file_item for file_item in all_resource_files if file_item.name == file_name]
			if len(file) > 0:
				suffix = file[0].suffix
				if suffix in ['.jpg', '.jpeg', '.png']:
					content_type = f'image/{suffix.lstrip(".")}'
				elif suffix in ['.pdf']:
					content_type = f'application/{suffix.lstrip(".")}'
				elif suffix in ['.xlsx']:
					content_type = 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'
				elif suffix in ['.xls']:
					content_type = 'application/vnd.ms-excel'
				else:
					content_type = '*/*'

				files_buffer.append(('file', (file_name, open(file[0].resolve(), 'rb'), content_type)))
			else:
				continue
		return files_buffer


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2023/4/20 16:06
# @Author  : yaitza
# @Email   : yaitza@foxmail.com

from dg_itest.servers.test_handler.test_file import *

def pytest_collect_file(parent, path):
    if path.ext in [".yml", ".xls", ".xlsx"] and path.basename.startswith("test"):
        return TestFile.from_parent(fspath=path, parent=parent)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2023/12/14 10:28
# @Author  : yaitza
# @Email   : yaitza@foxmail.com
import json
import time
import allure
from functools import wraps
from loguru import logger
from pathlib import Path


def init_log_config(log_dir: str, log_level):
	"""
	init log config
	:param log_dir: log locate position
	:param log_level: log show level
	:return:
	"""
	log_file_path = Path(log_dir).joinpath('{time:YYYYMMDD}.log')
	logger.add(log_file_path.absolute(),
	           format="{time:YYYY-MM-DD HH:mm:ss.SSS} | {level} | {name}:{function}:{line} - {message}",
	           rotation="00:00", encoding='utf-8', colorize=False, level=log_level)


def log_attach(log_data: str, name='compare_result'):
	"""
	log attach
	:param log_data:
	:return:
	"""
	logger.debug(log_data)
	allure.attach(log_data, name=name)


def allure_log(func):
	@wraps(func)
	def wrapper(*args, **kwargs):
		try:
			start = 1000 * time.time()
			logger.debug(f"========================  Begin: {func.__name__}  =================================")
			logger.debug(str(kwargs))
			allure.attach(str(kwargs), name='kwargs')
			result = func(*args, **kwargs)

			if hasattr(result, 'json'):
				logger.debug(str(result.json()))
				allure.attach(json.dumps(result.json(), sort_keys=True, indent=4, separators=(',', ':')).encode().decode("unicode_escape"), name='response')
			else:
				logger.debug(str(result))
				allure.attach(str(result).encode().decode("unicode_escape"), name='response')
			end = 1000 * time.time()
			logger.debug(f"=============   End: {func.__name__}; cost time: {end - start}ms   =============")
			return result
		except Exception as e:
			logger.error(repr(e))
			raise e

	return wrapper



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2024/2/21 16:05
# @Author  : yaitza
# @Email   : yaitza@foxmial.com
import time
import random

class KeyWords:
	def currenttime(self):
		"""
		获取当前时间
		:return: 当前时间, YYYYMMDDHHMMSS
		"""
		current_time = time.strftime('%Y%m%d%H%M%S', time.localtime(time.time()))
		return current_time

	def randomnum(self):
		"""
		获取随机数
		:return: 随机数, 10000-99999
		"""
		random_num = random.randint(10000, 99999)
		return str(random_num)


if __name__ == '__main__':
	change = KeyWords().randomnum()
	print(change, type(change))
	current_time = KeyWords().currenttime()
	print(current_time, type(current_time))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2023/4/20 11:16
# @Author  : yaitza
# @Email   : yaitza@foxmail.com

__all__ = ["DgItest"]

import os
import shutil
import sys
from pathlib import *
from dg_itest.utils.logger import logger, init_log_config
from dg_itest.utils.env_generater import EnvGenerater

import pytest


host_url = ""
local_test_res = ""

class DgItest:
    '''
    接口自动化测试框架
    '''
    def __init__(self, url, test_src, test_res, test_log_level='INFO'):
        '''
        url: 测试地址Host
        test_src: 测试用例归档目录
        test_res: 测试资源归档目录
        '''
        global host_url
        host_url = url
        global local_test_res
        local_test_res = test_res
        self.test_src = test_src
        self.test_log_level = test_log_level


    def __generate_conftest__(self, test_src):
        if Path(f'{test_src}/conftest.py').exists():
            return

        python_code ='''
#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2023/4/20 16:06
# @Author  : dg-itest
# @Email   : yaitza@foxmail.com

from dg_itest.servers.test_handler.test_file import *

def pytest_collect_file(parent, path):
    if path.ext in [".yml", ".xls", ".xlsx"] and path.basename.startswith("test"):
        return TestFile.from_parent(fspath=path, parent=parent)
        '''

        with Path(f'{test_src}/conftest.py').open(mode='w') as f:
            f.write(python_code)

    def run(self):
        init_log_config('./log', self.test_log_level)
        self.__generate_conftest__(self.test_src)

        test_report = 'test_report/result/'

        if Path(test_report).exists():
            shutil.rmtree(Path(test_report).parent.as_posix(), ignore_errors=True)
        if not Path(test_report).exists():
            Path(test_report).mkdir(parents=True)


        pytest.main(['-s', self.test_src, '--alluredir', Path(test_report).resolve().as_posix()])

        try:
            # 生成allure report中environment相关配置
            EnvGenerater.generate_env_file(host_url, Path(test_report))

            # 本地生成allure report 使用; 本地生成报告使用，commit时请注释
            os.system("allure generate ./test_report/result/ -o ./test_report/report --clean")
        except Exception:
            logger.error("请确保本地安装有allure，参考：https://docs.qameta.io/allure/")

if __name__ == '__main__':
    url = sys.argv[0]
    test_src = sys.argv[1]
    test_res = sys.argv[2]
    if  len(sys.argv) == 4:
        test_log_level = sys.argv[3]
    else:
        test_log_level = 'INFO'

    DgItest(url, test_src, test_res, test_log_level).run()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2023/4/21 14:44
# @Author  : yaitza
# @Email   : yaitza@foxmail.com
import json
import re

import jsonpath

from dg_itest.utils.cache import local_cache
from dg_itest.utils.keywords import KeyWords


class Replace:
    @staticmethod
    def replace_local_cache(source, pattern_str=r'\$.*?\$'):
        """
        替换本地缓存中的值
        :param source: 源数据
        :param pattern_str: 匹配规则
        :return: 替换后的数据
        """
        source_str = json.dumps(source)
        pattern = re.compile(pattern_str, re.DOTALL)
        match_items = pattern.findall(source_str)

        update = {}
        for items in match_items:
            update.update({items: local_cache.get(items)})
        if len(match_items) > 0:  # 一次只能替换一个值，故几个待替换的值就需要几次替换
            source = Replace.replace_exec(pattern_str, source, update)
        return source

    @staticmethod
    def replace_keyword(source, pattern_str=r'\#.*?\#'):
        """
        替换数据中的关键字，替换为关键字对应方法执行的值
        :param source: 源数据
        :param pattern_str: 匹配规则
        :return: 替换后的数据
        """
        source_str = json.dumps(source)
        pattern = re.compile(pattern_str, re.DOTALL)
        match_items = list(set(pattern.findall(source_str)))
        update = {}
        for items in match_items:
            func_name = items.replace('#', '')
            func_object = getattr(KeyWords(), func_name)
            update.update({items: func_object()})
        if len(match_items) > 0:  # 一次只能替换一个值，故几个待替换的值就需要几次替换
            source = Replace.replace_exec(pattern_str, source, update)
        return source

    @staticmethod
    def replace_exec(pattern: str, source, update : dict):
        """
        根据pattern匹配的值，在update中查找对应的值，并替换到source中
        :param pattern: 匹配规则
        :param source: 源数据
        :param update: 替换的值
        :return: 替换后的数据
        """
        if isinstance(source, str):
            result = Replace.replace_str(pattern, source, update)
        elif isinstance(source, dict):
            result = Replace.replace_dict(pattern, source, update)
        else:
            result = source
        return result


    @staticmethod
    def replace_str(pattern: str, source: str, update: dict) -> str:
        """
        替换源数据为字符串时替换方法
        :param pattern: 匹配规则
        :param source: 源数据
        :param update: 替换的值
        :return: 替换后的数据认为字符串
        """
        re_pattern = re.compile(pattern, re.DOTALL)
        match_results = list(set(re_pattern.findall(source)))

        for item in match_results:
            source = source.replace(item, update.get(item))
        return source

    @staticmethod
    def replace_dict(pattern: str, source: dict, update) -> dict:
        """
        替换源数据为字典时替换方法
        :param pattern: 匹配规则
        :param source: 源数据
        :param update: 替换的值
        :return: 替换后的数据仍为字典
        """
        re_pattern = re.compile(pattern, re.DOTALL)
        match_results = re_pattern.findall(json.dumps(source))
        result = {}
        for match_item in match_results:
            result = replace(source, match_item, update.get(match_item))
        match_results = re_pattern.findall(json.dumps(result))
        if match_results:
            for match_item in match_results:
                result_str = json.dumps(result).replace(match_item, str(update.get(match_item)))
                result = json.loads(result_str)
        return result

def replace(source: dict, match_item: str, update) -> dict:
    """
    递归替换字典中各个层级的值
    :param source: 源数据
    :param match_item: 匹配的值
    :param update: 替换的值
    :return: 替换后的数据
    """
    for k, v in source.items():
        if isinstance(v, dict):
            source[k] = replace(v, match_item, update)
        elif isinstance(v, list):
            if match_item in v:
                match_index = v.index(match_item)
                v[match_index] = update
                source.update({k: v})
            else:
                for v_item in v:
                    if isinstance(v_item, dict):
                        source[k][v.index(v_item)] = replace(v_item, match_item, update)
        elif isinstance(v, str):
            if v == match_item:
                source.update({k: update})
            elif v.__contains__(match_item):
                replace_v = v.replace(match_item, str(update))
                source.update({k: replace_v})
            else:
                pass
        else:
            pass

    return source

def replace_key(source: dict, match_item: str, update)-> dict:
    """
    递归替换字典中各个层级的key
    :param source: 源数据
    :param match_item: 匹配的值
    :param update: 替换的值
    :return: 替换后的数据
    """
    for k, v in source.items():
        if isinstance(v, dict):
            source[k] = replace(v, match_item, update)
        else:
            pass
    return source


if __name__ == '__main__':
    update = {'$file_period_map$': 3320, '$set_date$': '2018-1-1'}
    source = {"json":{"task_id":"$financial_task_id$","table_type":"$financial_table_type$","template_id":"$template_id$","file_period_map":{"$file_period_map$":["$set_date$"]}}, "headers": { "Authorization": "Bearer $accessToken$" }}
    source = Replace.replace_exec(r'\$.*?\$', source, update)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2023/4/19 18:25
# @Author  : yaitza
# @Email   : yaitza@foxmail.com
import json
import time

import allure
import pytest
import jsonpath
import traceback
from dg_itest.utils.logger import logger, log_attach
from dg_itest.servers.dg_servers.dg_singleton import DgSingleton
from dg_itest.servers.dg_servers.dg_after_handler import DgAfterHandler
from dg_itest.utils.cache import local_cache
from dg_itest.utils.replace import Replace
from dg_itest.utils.file_helper import FileHelper
class TestItem(pytest.Item):
    def __init__(self, name, parent, values):
        super(TestItem, self).__init__(name, parent)
        self.name = name
        self.values = values

    @pytest.mark.xfail()
    def runtest(self):
        logger.info(f'execute case: {self.name}; url: {self.values.get("request").get("url")}')
        request_data = self.replace(self.values['request'])
        params = request_data.get("params")
        if "files" in params.keys():
            params.update({"files": FileHelper.replace_file_to_stream(params.get("files"))})
        request_data.pop("params")
        request_data.update(params)
        exec_flag, exec_count = False, 0
        try:
            while exec_count < 10 and not exec_flag :  # 异步任务，轮询执行10次保证完成后进行校验
                api = DgSingleton().apis
                response = api.http_request(**request_data)
                DgAfterHandler().after(self.values, response)
                exec_flag = DgAfterHandler().check_status(self.values)
                if exec_count > 0 or not exec_flag:
                    log_attach(f'休眠30秒后，开始第{exec_count + 2}循环执行中...')
                    time.sleep(30)
                exec_count += 1
        except Exception:
            logger.error(traceback.format_exc())
            allure.attach(traceback.format_exc(), 'error details info')
            raise Exception(f'用例:{self.name} 执行失败，请检查！')

    def replace(self, source):
        """
        替换对应参数中的${},#{}
        """
        source = Replace.replace_local_cache(source)
        source = Replace.replace_keyword(source)
        return source


